#include <inttypes.h>
#include <stdio.h>
#include <stdlib.h>
#include <avr/interrupt.h>
#include <avr/signal.h>
#include <compat/ina90.h>


#include <avr/delay.h>

#include <avr/io.h>
#include <compat/twi.h>		/* Note [1] */


#include "ks0108.h"

#include "keypad.h"
#include "tools.h"


/*
#define DI   0
#define DO   1
#define CLK  2 
#define CS   3
*/
#define DI   3
#define DO   0
#define CLK  2 
#define CS   1
//#define INS  4
//#define WP   5

#define DATA_BLOCK_SIZE			64
#define SHIFTER_DATA_BLOCK		6

#define MMC_BLOCK_SIZE	512

char buffer [DATA_BLOCK_SIZE]={};			//Buffer


// *do not change unless you know what you're doing
// Value may be SIGNAL or INTERRUPT
#ifndef UART_INTERRUPT_HANDLER
#define UART_INTERRUPT_HANDLER	SIGNAL
#endif

#define UART_BAUD_RATE	4800L*4 // check FUSY bits! poseble x2 on exit

void uart_init(void)
{
//UBRRH = 0x00;					//Set 256000 bit per second
//UBRRL = 0x01;

UCSRB = /* BV(RXCIE)| BV(TXCIE)|*/ (1<<RXCIE)|(1<<RXEN)|(1<<TXEN);

unsigned short bauddiv = ((F_CPU+(UART_BAUD_RATE*8L))/(UART_BAUD_RATE*16L)-1);
UBRRL = bauddiv;
UCSRA = (1<<U2X);

asm("nop");

/*
UCSRA = 0x00;
UCSRB = (1<<RXEN)|(1<<TXEN);	//Receive and transmit enable
*/
//UCSRC = (1<<URSEL)|(1<<UCSZ1)|(1<<UCSZ0);	

}
/*
volatile unsigned char r_index=0, s_index=0;
//! UART Receive Complete Interrupt Handler
UART_INTERRUPT_HANDLER(SIG_UART_RECV)
{
	buffer[r_index] = UDR;
	if( ++r_index >= DATA_BLOCK_SIZE )
	{
		r_index = 0;
	}
}
*/

void uart_transmit(unsigned char data)	//transmit byte by UART
{

while ( !( UCSRA & (1<<UDRE)) )
; 
UDR = data;
}

unsigned char uart_receive (void)		//receive byte by UART
{ 

  while ( !(UCSRA & (1<<RXC)) )
  ;
  return UDR;
}

void uart_transmit_message_P(PGM_P str)	//transmit string by UART
{
	while(pgm_read_byte(str) != 0) {
		uart_transmit(pgm_read_byte(str++));
		}
}

void uart_transmit_message(char* msg)	//transmit string by UART
{ unsigned char i;
  i=0;

  while ((i<128)&(msg[i]!=0x00) )
   {
    uart_transmit(msg[i]);
    i++;
   }
}

void spi_transmit (unsigned char data)	//transmit byte to SD|MMC card
{ unsigned char i;

 for (i=0;i<8;i++)
  {
   if ((data&0x80)==0x00) PORTB&=~_BV(DI);		//Set data bit
                     else PORTB|=_BV(DI);
   data=data<<1; 
   PORTB|=_BV(CLK);								//Strobe
   asm("nop"); 
   PORTB&=~_BV(CLK);
  }
}

unsigned char spi_receive (void)		//receive byte from SD|MMC card
{ unsigned char i, res=0;

 for (i=0;i<8;i++)
  {
   PORTB|=_BV(CLK);								//Start strobe
   res=res<<1;
   if ((PINB&_BV(DO))!=0x00) res=res|0x01;		//Get data bit
   PORTB&=~_BV(CLK);							//End strobe
   asm("nop");
  }
 return res;
}

unsigned char sd_cmd(char b0, char b1, char b2, char b3, char b4, char b5)	//Send command to SD|MMC card
{ unsigned char res;
  long int count;
 spi_transmit (b0);		//transmit index

 spi_transmit (b1);		//transmit argument
 spi_transmit (b2);
 spi_transmit (b3);
 spi_transmit (b4);

 spi_transmit (b5);		//transmit CRC
 count=0;
 do {					//wait to R1 responce
     res=spi_receive();
     count=count+1; 
    } while ( ((res&0x80)!=0x00)&&(count<0xffff) ); 

 return res;
}

unsigned char sd_card_init(void)
{ unsigned char i,temp;
  long int count;

  for (i=0;i<10;i++)							//80 strobes
   spi_transmit (0xff);   
	
  PORTB&=~_BV(CS);								//CS enable
	
  temp=sd_cmd (0x40,0x00,0x00,0x00,0x00,0x95);	//CMD0
  if (temp!=0x01) return 3;						//Exit, if bad responce
  spi_transmit (0xff); 

  count=0;
  do{
     temp=sd_cmd (0x41,0x00,0x00,0x00,0x00,0x95);	//CMD1
     spi_transmit (0xff); 
	 count=count+1;
  } while ( (temp!=0x00)&&(count<0xffff) );			//Wait 0x01 responce

 if (count>=0xffff) return 4;                
return 0;
}

unsigned char read_block (char* buff, unsigned long int addr)
{ 
  unsigned char temp, buff_idx;
  unsigned long int count;
  unsigned short int in_block_offs, j; 

  unsigned char a1 = addr >> 24;
  unsigned char a2 = addr >> 16;
  unsigned char a3 = addr >> 8 & 0xFE;	// 512 mask

  temp=sd_cmd (0x51,a1,a2,a3,0x0,0x95);		//Send CMD17
  if (temp!=0x00) return 5;					//Exit, if responce not 0x00
  spi_transmit (0xff); 
  count=0;
  do{										//Wait to data block starts
    temp=spi_receive(); 
    count++;
   } while ( (temp!=0xfe)&&(count<0xffff) );

  if (count>=0xffff) return 5;

  in_block_offs = addr % MMC_BLOCK_SIZE;
 
  for(j=0, buff_idx=0; j<MMC_BLOCK_SIZE; j++)
	{
		if( in_block_offs <= j && (in_block_offs + DATA_BLOCK_SIZE) > j )
		{
			buff[buff_idx] = spi_receive();
			buff_idx++;
		}
		else
			spi_receive();
	}

 spi_receive();								//receive CRC
 spi_receive();

 return 0;
}


#define FIRST_BLOCK_WRITE	1
#define LAST_BLOCK_WRITE	3
/*
unsigned char write_block (char* buff, unsigned long int addr, char flg)
{
  unsigned char temp;
  unsigned char a1, a2, a3;
  long int count; 
  short int buff_idx; 
 if (FIRST_BLOCK_WRITE == flg){
  a1 = addr >> 24;
  a2 = addr >> 16;
  a3 = addr >> 8;

   temp=sd_cmd(0x58,a1,a2,a3,0x0,0x95);		//Send CMD24
   if (temp!=0x00) return 6;					//Exit, if responce non 0x00
   spi_transmit (0xff); 

   spi_transmit (0xfe);						//Start data block
 }

  uart_transmit('\10');
  uart_transmit('\13');
  uart_transmit(r_index+'0');
  uart_transmit('\10');
  uart_transmit('\13');
  uart_transmit(s_index+'0');
  
  while( 1 )
	{
		if( r_index != s_index )
		{
			spi_transmit(buffer[s_index]);
			s_index++;
			if( s_index >= DATA_BLOCK_SIZE )
			{
				s_index = 0;
				break;
			}
		}
	}
 
  if (LAST_BLOCK_WRITE == flg){
    spi_transmit (0xff);						//Send CRC
    spi_transmit (0xff);
    temp=spi_receive();
    if ((temp&0x05)!=0x05) return 6;			//Exit, if data responce non 0x05

    count=0;
    do {										//Wait end of busy state responce
      temp=spi_receive();
      count++;
    }while ( (temp!=0xff)&&(count<0xffff) );

    if (count>=0xffff) return 6;
  }
 return 0;
}
  
  */
/*  
  //
  //
  //
  int b_to_hex( unsigned char c, char *p)
  {
   unsigned char h = c >> 4;
   unsigned char l = c & 0x0F;
   
   if ( h > 9 )
       *p = h - 10 + 'A';
	else
       *p = h + '0';	   
	p++;
	
   if ( l > 9 )
       *p = l - 10 + 'A';
	else
       *p = l + '0';	   
	p++;
  
	*p = '\0';
	
  return 0;
  }

  //
  //
  //
  int b_to_dec( char c, char *p)
  {
    *p = (c/100) + '0';
	p++;
    *p = (c%100)/10 + '0';
	p++;
    *p = (c%10) + '0';
	p++;
	*p = '\0';
  return 0;
  }
  */
/*
///
//	dont forgot will do enable interupt
//
int write_dict(unsigned int long first_block, unsigned long int cnt_blocks)
{
  unsigned char temp,ch;
  short int i;
  unsigned long int offs;

  for(offs=first_block*MMC_BLOCK_SIZE; offs<(MMC_BLOCK_SIZE*cnt_blocks*first_block); offs+=MMC_BLOCK_SIZE)
  {
					// uart_transmit_message("write_block\r\n");

                  temp=write_block(buffer, offs, FIRST_BLOCK_WRITE);		//write buffer
				  if(temp)
				    {
					  ;//uart_transmit_message("write_block: CMD24 fail\r\n");
					  return 6;
					}

					
                  for (i=0;i<(MMC_BLOCK_SIZE/DATA_BLOCK_SIZE)-2L;i++)  // because FIRST_BLOCK_WRITE,LAST_BLOCK_WRITE
				  {
					temp=write_block(buffer, offs, 0);		//write buffer
                    if(temp==0x00) 
					  ;//uart_transmit_message("write_block: block writte succes\r\n");
				    else
					{				  
				      uart_transmit_message("write_block: CMD24 fail\r\n");
					  return 6;
					}
				  }
					
				  
					ch = buffer[0];
					temp=write_block(buffer, offs, LAST_BLOCK_WRITE);		//write buffer
                  if(temp==0x00) 
					;//uart_transmit_message("write_block: block writte succes\r\n");
				  else
					{				  
				      uart_transmit_message("write_block: CMD24 fail\r\n");
					  return 6;
					}
			uart_transmit(ch);
  }
  return 0;
}

*/

#define FIND_FIRST_POSITION		1
#define FIND_LAST_POSITION		2

#define SEARCH_STR_LEN	6

#define FIRST_POS	0
#define LAST_POS	1
#define INPUT_CH	2

unsigned long int first_last_position[3][SEARCH_STR_LEN];		// !!! realy 6


unsigned long int find_char(unsigned char idx, char x, char flg_position)
{
  unsigned char temp;
 // int i;
  unsigned long int first, last;
  unsigned long int mid;
  char a;

  first = first_last_position[FIRST_POS][idx];
  last  = first_last_position[LAST_POS][idx];
  
	while ( first < last ) {
		mid = ( first + last ) >> 1;
		
		temp=read_block(buffer, (mid << SHIFTER_DATA_BLOCK));
                if(temp==0x00)
				{
				    //uart_transmit_message("read_block: read succes:\r\n");
				    //for (i=0;i<DATA_BLOCK_SIZE;i++)				//Send buffer to UART
					//uart_transmit(buffer[i]);
				}
				else 
				    uart_transmit_message("read_block: CMD17 fail\r\n");
		a = buffer[idx];
		// uart_transmit(a);	//transmit byte by UART
		switch( flg_position )
		{
			case FIND_FIRST_POSITION:
				if ( x <= a ) {        
					last = mid;
				} else {
					first = mid + 1;
				}
				break;
			case FIND_LAST_POSITION:
				if ( x < a ) {
					last = mid;
				} else {
					first = mid + 1;
				}
				break;
		}
	}
	uart_transmit('\n');	//transmit byte by UART
	uart_transmit('\r');	//transmit byte by UART
	return last;
}

#define SOCRAT_DICT_KEYWORD_BLOCK_OFFS		1L
#define SOCRAT_DICT_KEYWORD_BLOCK_SIZE		7042L
#define SOCRAT_DICT_ARTICLE_BLOCK_OFFS		7101L
#define SOCRAT_DICT_ARTICLE_BLOCK_SIZE		14084L

extern char keyState;

void main(void) __attribute__((__noreturn__));
void main(void)
{ 
  unsigned char temp; 
  int i;
  unsigned char idx_char=0;
  char ch=0, key=0;

	// Wait a little while the display starts up
	for(volatile uint16_t i=0; i<15000; i++);
	
	// Initialize the LCD
	ks0108Init(0);
	
	// Select a font
	//ks0108SelectFont(SC, ks0108ReadFontData, BLACK);
	// Set a position
	ks0108GotoXY(0,0);
	// Print some text
	ks0108Puts_P(PSTR("AVR diction"));
	// a nice little round rect
	//ks0108DrawRoundRect(1, 1, 126, 30, 6, BLACK);
    // Set a position
	//ks0108GotoXY(10,5);

	// Once again :)
	// Select a font
	//ks0108SelectFont(SC, ks0108ReadFontData, BLACK);
	//while(1);
 
   first_last_position[FIRST_POS][0] = MMC_BLOCK_SIZE / DATA_BLOCK_SIZE;
   first_last_position[LAST_POS][0] =  56336L;//  7042 * 512/DATA_BLOCK_SIZE;
  
  PORTB=_BV(CS)|_BV(DO)|_BV(DI) /*|_BV(WP)|_BV(INS)*/; 	//Init ports
  DDRB=_BV(CS)|_BV(DI)|_BV(CLK);

  uart_init();			//Init UART
  keypad_init();
  sei();

 uart_transmit_message_P(PSTR("\n\n\n\nuart_init: ok\r\n"));
 temp=sd_card_init();	//Init card

 
 if (temp==0x00) 
	{
                  //for (i=0;i<DATA_BLOCK_SIZE;i++)
				  //    buffer[i]=0x30+(i%10);		//Set buffer 12345...
/*
				__asm__ __volatile__ ("sei" ::);
				nop;nop;nop;
				temp = write_dict(1, 7042L );	// by blocks in 512
                if(temp==0x00) 
				{
				    uart_transmit_message("succes:\r\n");
				}
				else 
				    uart_transmit_message("fail\r\n");
*/
/*
				temp = write_dict(7101L, 14084L );	// by blocks in 512
                if(temp==0x00) 
				{
				    uart_transmit_message("succes:\r\n");
				}
				else 
				    uart_transmit_message("fail\r\n");
				while (1) 
					;
*/
	ks0108DrawRect(0, 8, 127, 10, BLACK);

while(1){

while(1){
	_delay_ms(50);
	
	key = getKeyChar();

	if( keyState & CHAR_IS_READY )
	{
		keyState &= ~CHAR_IS_READY;
		break;
	}
	else
	{
		if(key){
			ks0108FillRect(2 + idx_char*(FONT_FIXED_WIDTH+1), 10, FONT_FIXED_WIDTH*2+2 /* 2 symb clear */, FONT_HEIGHT, WHITE);
			ks0108GotoXY(2 + idx_char*(FONT_FIXED_WIDTH+1), 10);
			ks0108PutChar(key);
			}
	}
}
	
		if (key=='#')
			key = 8;
		//key = uart_receive();
		ch = key;
		uart_transmit(ch);
		uart_transmit(idx_char+'0');
		if( key == 8 )
		{
			if(!idx_char)
			{
				ch = 'a';
			}
			else
			{
				idx_char--;
				ch = first_last_position[INPUT_CH][idx_char];
			}
		}

		// Clear 
		ks0108FillRect(0, 20, 21*FONT_FIXED_WIDTH+21, FONT_HEIGHT, WHITE);
		ks0108FillRect(0, 30, 21*FONT_FIXED_WIDTH+21, FONT_HEIGHT*3+3, WHITE);
		
		first_last_position[INPUT_CH][idx_char] = ch;
		first_last_position[FIRST_POS][idx_char+1] = find_char(idx_char, ch, FIND_FIRST_POSITION);
		
		first_last_position[LAST_POS][idx_char+1] = find_char(idx_char, ch, FIND_LAST_POSITION);

  		read_block(buffer, (first_last_position[FIRST_POS][idx_char+1] << SHIFTER_DATA_BLOCK));
		ks0108GotoXY(0, 20);
		for (i=0; i<21/* DATA_BLOCK_SIZE */; i++)				//Send buffer to UART
			{
				uart_transmit(buffer[i]);
			}
		buffer[i] = 0;
		ks0108Puts(buffer);

		uart_transmit_message_P(PSTR("\r\n"));
  		read_block(buffer, ((first_last_position[FIRST_POS][idx_char+1] << SHIFTER_DATA_BLOCK) + (7100L * MMC_BLOCK_SIZE) + \
		((first_last_position[FIRST_POS][idx_char+1] - (MMC_BLOCK_SIZE / DATA_BLOCK_SIZE)) << SHIFTER_DATA_BLOCK)));
		ks0108GotoXY(0, 30);
		for (i=0;i<DATA_BLOCK_SIZE;i++)				//Send buffer to UART
			{
				uart_transmit(buffer[i]);
				if(buffer[i]>=0xe0)
				{
					buffer[i] -= 65;
				}
				if(21==i)
				{
					ks0108GotoXY(0, 40);
				}
				ks0108PutChar(buffer[i]);
			}
		uart_transmit_message_P(PSTR("\r\n"));

		if( key != 8 )
			idx_char++;
	}
	}

 else if (temp==0x03) uart_transmit_message_P(PSTR("sd_card_init: CMD0 fail\r\n"));
 else uart_transmit_message_P(PSTR("sd_card_init: CMD1 fail\r\n"));

 while (1) 
  ;

// return 1; !!!
}
