#include <avr/interrupt.h>
#include <util/delay.h>

#include <inttypes.h>
#include <avr/io.h>
#include <avr/pgmspace.h>
#include "keypad.h"
#include "tools.h"


void keypad_init(void)
{
// FCPU / 1024
	TCCR0 |= (1<<CS02)|(1<<CS00);
	TIMSK |= (1 << TOIE0) ; // setup timer 0 to interrupt on overflow 
	TCNT0 = 0;

	DDRC=0xF0;
	PORTC=0x0F;

	ACSR=0x80;

	PORTC = 0xFF;
}

static unsigned char __attribute__ ((progmem)) tableSymbIdx[] = { 
					6, 18, 31, 40,
					2, 14, 27, 38,
					0, 10, 22, 36
					    };

static unsigned char __attribute__ ((progmem)) tableSymb[] = {   
					' ','\0',                'a','b','c','\0',    'd','e','f','\0',
				    'g','h','i','\0',        'j','k','l','\0',    'm','n','o','\0',
					'p','k','r','s','\0',    't','u','v','\0',    'w','x','y','z','\0',
					'*','\0', 			      ' ','\0',		       '#','\0'
					                     };
 

volatile char keyIdx=0;
volatile char symbIdx=0;
volatile char keyState=0;
volatile char interuptCounter=0;
volatile char keyboardChar=0;
volatile char tempChar=0;

					  
ISR (TIMER0_OVF_vect)
{

  char key = 0;
  char i=0, j=0;
  TIMSK &= ~(1 << TOIE0) ; // clear timer 0 to interrupt on overflow 
//		uart_transmit('.');

	keyState &= ~KEY_PRESSED;
	for (i=0; i<4 ;i++)
	{
	   PORTC = ~(0x10 << i);	// PORT <- value
	   for (j=0;j<4;j++)
	   {
		  if (bit_is_clear(PINC, j))
				   key=j;
		}
		_delay_ms(2);
		if (bit_is_clear(PINC, key))
		   {
//			    uart_transmit('\r'); uart_transmit('\n');
//			    uart_transmit('i');	uart_transmit('=');
//			    uart_transmit((key + 4 * i)+'0');

				keyIdx = key + (i << 2);
				keyState |= KEY_PRESSED;
		   }
		PORTC = ~(0x10 << i);
	}

	

  TIMSK |= (1 << TOIE0) ; // setup timer 0 to interrupt on overflow 
  TCNT0 = 0;
}
char getKeyChar(void)
{
static char lastChar=0;
static char currCh=0;
static char ch = 0, idx = 0;
static char shift = 0;

	if (interuptCounter++ > 30)
	{
			tempChar = ch;
			if( interuptCounter > 60 )
			{
			if( ch && tempChar == ch )
				{
				keyboardChar = tempChar;
				keyState |= CHAR_IS_READY;
				ch = 0;
				
//				uart_transmit('\r');
//				uart_transmit('\n');
//				uart_transmit('k');
//				uart_transmit('=');
//				uart_transmit(keyboardChar);
				}
				interuptCounter = 0;
				return tempChar;
			}
	}


//	_delay_ms(100);

	if( keyState & KEY_PRESSED )
	{
		idx = pgm_read_byte(&tableSymbIdx[keyIdx]);
		lastChar = pgm_read_byte(&tableSymb[idx]);
	}
	else
	{
		lastChar = 0;
	}
	
	if( lastChar )
		{
		if( currCh != lastChar )
		   {
		   interuptCounter = 0;
			ch = pgm_read_byte(&tableSymb[idx+shift++]);
			if(!ch)
			{
				shift=0;
				ch = pgm_read_byte(&tableSymb[idx+shift++]);
			}


//			uart_transmit('\r');
//			uart_transmit('\n');
//			uart_transmit('c');
//			uart_transmit('>');
//			uart_transmit(ch);
			
			  
			}
		   currCh = lastChar;

		}
		else
		{
		  currCh = 0;
		  //shift = 0;
		}
return ch;
}
/*
int main(void)
{

char state=0;
char ch =0;


  while (1){
	_delay_ms(100);
	
	ch = getKeyChar();

		uart_transmit('\r');
		uart_transmit('\n');
	if( keyState & CHAR_IS_READY )
	{
		keyState &= ~CHAR_IS_READY;
		uart_transmit('i');
		uart_transmit('=');
		uart_transmit(ch);
	}
	else
	{
		uart_transmit('c');
		uart_transmit('=');
		uart_transmit(ch);
	}
  }
}
*/